// Copyright (c) 2016-2024 CloudCrane. All rights reserved.
//
// This work is licensed under the MIT License.
// See the LICENSE file in the project root for more information.

package cmd

import (
	"errors"
	"os"

	"github.com/spf13/cobra"
	xbus "xbus.io/go-xbus/v4"
	"xbus.io/go-xbus/v4/api"
)

// actorDeleteCmd represents the actor delete sub-command
var actorDeleteCmd = &cobra.Command{
	Use:   "delete",
	Short: "Delete an actor",
	Long: "Delete an actor from the xbus server. You must restart " +
		"the xbus clients for this change to be taken into account.",
	Run: func(cmd *cobra.Command, args []string) {
		if len(args) != 1 {
			Logger.Error().Msgf("expects one and only one identifier")
			usage(cmd)
		}
		os.Exit(doDeleteActor(args[0]))
	},
}

func doDeleteActor(identifier string) int {
	conn, err := GetClient(false)
	if err != nil {
		Logger.Error().Msgf("Cannot connect to the NATS server: %s", err)
		return 1
	}
	actor, err := conn.ActorFind(identifier)
	if err != nil {
		if errors.Is(err, xbus.ErrNotFound) {
			Logger.Error().Msgf("actor not found")
		} else {
			Logger.Error().Msgf("could not find actor: %s", err)
		}
		return -1
	}
	if err := conn.ActorDelete([]api.UUID{actor.GetIDAsUUID()}); err != nil {
		Logger.Error().Msgf("could not delete actor: %s", err)
		return -1
	}
	return 0
}

func init() {
	actorCmd.AddCommand(actorDeleteCmd)
}
