// Copyright (c) 2016-2024 CloudCrane. All rights reserved.
//
// This work is licensed under the MIT License.
// See the LICENSE file in the project root for more information.

package main

import (
	"math/rand"
	"time"

	_ "github.com/lib/pq"
	_ "github.com/orus-io/qb/dialects/postgres"

	"xbus.io/xbus/cmd/xbusd/cmd"
	"xbus.io/xbus/utils/license"
)

func init() {
	// TODO rand.Seed is deprecated

	// SA1019: rand.Seed has been deprecated since Go 1.20 and an alternative
	// has been available since Go 1.0: Programs that call Seed and then expect
	// a specific sequence of results from the global random source (using
	// functions such as Int) can be broken when a dependency changes how much
	// it consumes from the global random source. To avoid such breakages,
	// programs that need a specific result sequence should use NewRand
	// (NewSource(seed)) to obtain a random generator that other packages
	// cannot access. (staticcheck)

	rand.Seed(time.Now().UnixNano()) //nolint: staticcheck
}

func main() {
	license.DisplayLicenseOnStart()
	license.SetupCmd(cmd.RootCmd)
	cmd.Execute()
}
