// Copyright (c) 2016-2024 CloudCrane. All rights reserved.
//
// This work is licensed under the MIT License.
// See the LICENSE file in the project root for more information.

package fullstack

import (
	"context"
	"testing"
	"time"

	"github.com/stretchr/testify/assert"

	"xbus.io/xbus/testutils"
	test_storage "xbus.io/xbus/testutils/storage"
)

func TestEnvelopeStorageService(t *testing.T) {
	ctx := testutils.GetTestContext(t)
	ft := InitFullstack(t, true)
	if ft == nil {
		return
	}
	defer ft.Shutdown()

	envelopeStorage := ft.Server.GetEnvelopeStorage()

	fragments := test_storage.GenBigEnvFragments()
	for _, fragment := range fragments {
		_, err := envelopeStorage.StoreEnvelope(ctx, fragment)
		if err != nil {
			t.Fatal(err)
		}
	}
	lastFragment := fragments[len(fragments)-1]

	er := ft.Client.RetrieveEnvelope(
		context.Background(), "service", "-", lastFragment.GetIDAsUUID(),
	)

	envelope, err := er.Complete(4 * time.Second)
	if err != nil {
		t.Fatal(err)
	}
	for _, event := range envelope.Events {
		switch event.Type {
		case "xbus.test":
			assert.EqualValues(t, 0xbdac9720, event.Checksum)
		case "xbus.test.2":
			assert.EqualValues(t, 0x560279ee, event.Checksum)
		default:
			t.Fatal("Unexpected event in the envelope")
		}
	}
}
