// Copyright (c) 2016-2024 CloudCrane. All rights reserved.
//
// This work is licensed under the MIT License.
// See the LICENSE file in the project root for more information.

package control

import (
	"context"
	"encoding/json"
	"time"

	"xbus.io/go-xbus/v4/api"
	"xbus.io/go-xbus/v4/api/control"
)

type pmprocessServer struct {
	*Service
}

func (s pmprocessServer) Query(
	ctx context.Context, req *control.PMProcessQueryRequest,
	send func(*control.PMProcessQueryReply),
) error {
	statuses := []api.PMProcess_Status{
		api.PMProcess_NEW,
		api.PMProcess_OPENED,
	}
	if req.IncludeClosed {
		statuses = append(statuses, api.PMProcess_CLOSED)
	}
	list, err := s.o.ProcessManager.ListPostMortem(ctx, req.Level, statuses...)
	if err != nil {
		panic(err)
	}
	var resp control.PMProcessQueryReply
	resp.PMProcessList = make([]*api.PMProcess, 0, 200)
	for _, process := range list {
		pmstate, err := process.GetPostMortemState(ctx)
		if err != nil {
			panic(err)
		}
		state := s.o.ProcessStorage.GetState(ctx, process.ID)
		b, err := json.Marshal(state)
		if err != nil {
			panic(err)
		}
		resp.PMProcessList = append(resp.PMProcessList,
			&api.PMProcess{
				Process: &api.Process{
					ID:                process.ID.ToBytes(),
					TriggerEmitterID:  process.TriggerEmitterID.ToBytes(),
					TriggerEnvelopeID: process.TriggerEnvelopeID.ToBytes(),
					PipelineID:        process.Pipeline.ID,
					Status:            process.GetStatus(ctx),
					State:             string(b),
				},
				Level:   pmstate.Level,
				Status:  pmstate.Status,
				Comment: pmstate.Comment,
			})
		if len(resp.PMProcessList) > 100 {
			send(&resp)
			resp.PMProcessList = make([]*api.PMProcess, 0, 200)
		}
	}
	send(&resp)
	return nil
}

func (s pmprocessServer) SetStatus(
	ctx context.Context, req *control.PMProcessSetStatusRequest,
) (err error) {
	process, err := s.o.ProcessManager.Get(ctx, req.GetProcessIDAsUUID())
	if err != nil {
		panic(err)
	}

	pmState, err := process.GetPostMortemState(ctx)
	if err != nil {
		return
	}

	pmState.Status = req.Status
	pmState.Comment = req.Comment
	pmState.Changed = time.Now()

	err = process.SetPostMortemState(ctx, pmState)
	return
}
