// Copyright (c) 2016-2024 CloudCrane. All rights reserved.
//
// This work is licensed under the MIT License.
// See the LICENSE file in the project root for more information.

package storage

import (
	"context"

	"xbus.io/go-xbus/v4/api"
)

// AccountStorage is in charge of storing account definitions.
// When multiple instances are running, CRUD operations be reflected from one
// to the others
type AccountStorage interface {
	Get(ctx context.Context, id api.UUID) (*api.Account, bool)
	GetByName(ctx context.Context, name string) (*api.Account, bool)
	GetByCert(ctx context.Context, cert string) (*api.Account, bool)
	GetByAPIKey(ctx context.Context, key string) (*api.Account, bool)

	Create(context.Context, *api.Account) (*api.Account, error)
	Update(context.Context, *api.Account) (*api.Account, error)
	Delete(context.Context, *api.Account)

	List(context.Context) []*api.Account
}
