// Copyright (c) 2016-2024 CloudCrane. All rights reserved.
//
// This work is licensed under the MIT License.
// See the LICENSE file in the project root for more information.

package storage

import (
	"context"
	"time"

	"xbus.io/go-xbus/v4/api"
)

// ActorEventHandler is the type for callbacks
type ActorEventHandler func(*api.Actor)

// NoopActorEventHandler is an ActorEventHandler that does nothing
func NoopActorEventHandler(*api.Actor) {}

// ActorStorage is in charge of storing the actor definitions
// When multiple instances are running, CRUD operations be reflected from one
// to the others
type ActorStorage interface {
	Get(context.Context, api.UUID) (*api.Actor, bool)
	GetByName(ctx context.Context, name string) (*api.Actor, bool)
	List(context.Context) []*api.Actor
	ListAccount(ctx context.Context, accountID api.UUID) []*api.Actor
	ListKind(ctx context.Context, actorKind api.Actor_Kind) []*api.Actor
	ListRole(ctx context.Context, actorKind api.Actor_Kind, role string) []*api.Actor

	Create(context.Context, *api.Actor) (*api.Actor, error)
	Delete(context.Context, api.UUID)
	Update(context.Context, *api.Actor) (*api.Actor, error)

	MarkUnresponsive(ctx context.Context, id api.UUID, decisionTime time.Time) error
	JustSeen(context.Context, api.UUID) error
	SetOnline(context.Context, api.UUID, bool) error

	SetEventHandlers(onActorUp, onActorDown ActorEventHandler)
}
