// Copyright (c) 2016-2024 CloudCrane. All rights reserved.
//
// This work is licensed under the MIT License.
// See the LICENSE file in the project root for more information.

package storage

import (
	jsoniter "github.com/json-iterator/go"
	"xbus.io/go-xbus/v4/utils"
)

func (g Graph) MarshalJSONStream(stream *jsoniter.Stream) {
	obj := utils.NewJSONStreamObject(stream)
	nodes := obj.Array("nodes")
	for _, node := range g.Nodes {
		nodes.Val(node)
	}
	edges := obj.Array("edges")
	for _, edge := range g.Edges {
		edges.String(edge.String())
	}
	obj.End()
}

func (n Node) MarshalJSONStream(stream *jsoniter.Stream) {
	obj := utils.NewJSONStreamObject(stream)
	obj.OEString("id", n.ID)
	obj.Val("type", n.Type)
	if len(n.ActorIDs) != 0 {
		a := obj.Array("actorids")
		for _, id := range n.ActorIDs {
			a.Val(id)
		}
		a.End()
	}
	obj.OEStringList("actors", n.Actors)
	obj.OEStringList("roles", n.Roles)
	if n.RoleBroadcast {
		obj.True("rolebroadcast")
	}
	if n.SourceMatch != nil {
		obj.Val("sourcematch", n.SourceMatch)
	}
	obj.OEStringList("inputs", n.Inputs)
	obj.OEStringList("outputs", n.Outputs)
	obj.End()
}

func (sm SourceMatch) MarshalJSONStream(stream *jsoniter.Stream) {
	obj := utils.NewJSONStreamObject(stream)
	obj.OEStringList("eventtypes", sm.EventTypes)
	obj.OEBool("byactor", sm.ByActor)
	obj.Val("priority", sm.Priority)
	obj.End()
}
