// Copyright (c) 2016-2024 CloudCrane. All rights reserved.
//
// This work is licensed under the MIT License.
// See the LICENSE file in the project root for more information.

package storage

import (
	"xbus.io/go-xbus/v4/api"
)

type Datatype string

func (d Datatype) String() string {
	return string(d)
}

const (
	DatatypeEnvelope Datatype = "envelope"
)

type NoSuchIDError struct {
	datatype Datatype
	id       api.UUID
}

func (err NoSuchIDError) Unwrap() error {
	return ErrNoSuchID
}

func (err NoSuchIDError) Error() string {
	return "No such " + err.datatype.String() + ": " + err.id.String()
}

func NewNoSuchIDError(datatype Datatype, id api.UUID) *NoSuchIDError {
	return &NoSuchIDError{
		datatype: datatype,
		id:       id,
	}
}
