package cmd

import (
	"context"
	"fmt"
	"os"

	"github.com/spf13/cobra"

	"xbus.io/go-xbus/v4"
	"xbus.io/go-xbus/v4/api"
)

var emitCmd = cobra.Command{
	Use:   "emit",
	Short: "Emit arbitrary data with an emitter",
	Run: func(cmd *cobra.Command, args []string) {
		if len(args) == 0 || len(args) > 2 {
			if err := cmd.Usage(); err != nil {
				fmt.Println("Error printing usage:", err)
			}
			os.Exit(1)
		}
		emitter := args[0]
		input := cmd.Flag("input").Value.String()
		os.Exit(doEmit(emitter, input))
	},
}

func doEmit(emitter, input string) int {
	client, err := GetClient(false)
	if err != nil {
		Logger.Err(err).Msg("Error initializing")
		return 1
	}

	if err := client.Startup(false); err != nil {
		Logger.Err(err).Msg("Error starting client")
		return 1
	}
	defer client.Shutdown()

	var actor *xbus.Actor
	if actorID, err := api.UUIDFromString(emitter); err == nil {
		actor = client.GetActor(actorID)
	} else {
		actor = client.GetActorByName(emitter)
	}
	if actor == nil {
		Logger.Err(err).Msg("No such emitter")
		return 1
	}

	envelope, err := loadEnvelope(input)
	if err != nil {
		Logger.Err(err).Msg("Error loading envelope")
		return 1
	}

	if _, err := actor.Emit(context.Background(), envelope); err != nil {
		Logger.Err(err).Msg("Error sending")
		return 1
	}

	return 0
}

func init() {
	flags := emitCmd.PersistentFlags()
	flags.StringP("input", "i", "-", "input file. Default is stdin ('-')")
	RootCmd.AddCommand(&emitCmd)
}
