package demo

import (
	"time"

	xbus "xbus.io/go-xbus/v4"
)

// NewRelayWorker is a RelayWorker factory
func NewRelayWorker(actor *xbus.Actor, settings xbus.Settings) (xbus.ActorService, error) {
	return &RelayWorker{
		time.Duration(settings.MustIntD("processing-duration", 1)) * time.Second,
	}, nil
}

// RelayWorker is a worker that forward incoming envelopes to the default output
// The service parameters are:
// - processing-duration: time to wait between receive and send, in seconds
type RelayWorker struct {
	interval time.Duration
}

// Process handles an incoming process
func (w *RelayWorker) Process(apc *xbus.ActorProcessingContext) error {
	select {
	case <-apc.Context.Done():
	case <-time.After(w.interval):
		if err := apc.ForwardInput("default", "default"); err != nil {
			apc.Log.Err(err).Msg("Something wrong happened.")
		} else {
			apc.Log.Info().Msg("Successfully forwarded the envelope")
		}
	}
	return nil
}

func init() {
	xbus.RegisterActorService("demo.relay", NewRelayWorker)
}
