package utils

import jsoniter "github.com/json-iterator/go"

func NewJSONStreamArray(stream *jsoniter.Stream) *JSONStreamArray {
	return &JSONStreamArray{
		stream:  stream,
		isEmpty: true,
	}
}

type JSONStreamArray struct {
	stream  *jsoniter.Stream
	isEmpty bool
}

func (sa *JSONStreamArray) End() {
	if sa.isEmpty {
		sa.stream.WriteEmptyArray()
	} else {
		sa.stream.WriteArrayEnd()
	}
}

func (sa *JSONStreamArray) Add() {
	if sa.isEmpty {
		sa.isEmpty = false
		sa.stream.WriteArrayStart()
	} else {
		sa.stream.WriteMore()
	}
}
func (sa *JSONStreamArray) Nil() {
	sa.Add()
	sa.stream.WriteNil()
}

func (sa *JSONStreamArray) True() {
	sa.Add()
	sa.stream.WriteTrue()
}

func (sa *JSONStreamArray) False() {
	sa.Add()
	sa.stream.WriteFalse()
}

func (sa *JSONStreamArray) Bool(value bool) {
	sa.Add()
	sa.stream.WriteBool(value)
}

func (sa *JSONStreamArray) Object() *JSONStreamObject {
	sa.Add()
	return NewJSONStreamObject(sa.stream)
}

func (sa *JSONStreamArray) Array() *JSONStreamArray {
	sa.Add()
	return NewJSONStreamArray(sa.stream)
}

func (sa *JSONStreamArray) StringWithHTMLEscaped(value string) {
	sa.Add()
	sa.stream.WriteStringWithHTMLEscaped(value)
}

func (sa *JSONStreamArray) String(value string) {
	sa.Add()
	sa.stream.WriteString(value)
}

func (sa *JSONStreamArray) Uint8(value uint8) {
	sa.Add()
	sa.stream.WriteUint8(value)
}

func (sa *JSONStreamArray) Int8(value int8) {
	sa.Add()
	sa.stream.WriteInt8(value)
}

func (sa *JSONStreamArray) Uint16(value uint16) {
	sa.Add()
	sa.stream.WriteUint16(value)
}

func (sa *JSONStreamArray) Int16(value int16) {
	sa.Add()
	sa.stream.WriteInt16(value)
}

func (sa *JSONStreamArray) Uint32(value uint32) {
	sa.Add()
	sa.stream.WriteUint32(value)
}

func (sa *JSONStreamArray) Int32(value int32) {
	sa.Add()
	sa.stream.WriteInt32(value)
}

func (sa *JSONStreamArray) Uint64(value uint64) {
	sa.Add()
	sa.stream.WriteUint64(value)
}

func (sa *JSONStreamArray) Int64(value int64) {
	sa.Add()
	sa.stream.WriteInt64(value)
}

func (sa *JSONStreamArray) Uint(value uint) {
	sa.Add()
	sa.stream.WriteUint(value)
}

func (sa *JSONStreamArray) Int(value int) {
	sa.Add()
	sa.stream.WriteInt(value)
}

func (sa *JSONStreamArray) Float32(value float32) {
	sa.Add()
	sa.stream.WriteFloat32(value)
}

func (sa *JSONStreamArray) Float32Lossy(value float32) {
	sa.Add()
	sa.stream.WriteFloat32Lossy(value)
}

func (sa *JSONStreamArray) Float64(value float64) {
	sa.Add()
	sa.stream.WriteFloat64(value)
}

func (sa *JSONStreamArray) Float64Lossy(value float64) {
	sa.Add()
	sa.stream.WriteFloat64Lossy(value)
}

func (sa *JSONStreamArray) Val(value any) {
	sa.Add()
	if m, ok := value.(JSONStreamMarshaler); ok {
		m.MarshalJSONStream(sa.stream)
	} else {
		sa.stream.WriteVal(value)
	}
}
